# Generated by Django 2.1.4 on 2018-12-10 11:05

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import django_extensions.db.fields
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Company',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(help_text='company name - must be unique', max_length=200, unique=True, verbose_name='name')),
                ('slug', django_extensions.db.fields.AutoSlugField(blank=True, editable=False, populate_from='name')),
                ('address', models.CharField(help_text='address', max_length=64, verbose_name='address')),
                ('contact_name', models.CharField(help_text='contact name', max_length=30, verbose_name='contact name')),
                ('contact_phone', models.CharField(help_text='contact phone number', max_length=30, verbose_name='contact phone')),
            ],
            options={
                'db_table': 'pyfb_company',
                'verbose_name': 'company',
                'ordering': ('name',),
                'verbose_name_plural': 'companies',
            },
        ),
        migrations.CreateModel(
            name='CompanyBalanceHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('amount_debited', models.DecimalField(decimal_places=4, max_digits=12, verbose_name='amount debited')),
                ('amount_refund', models.DecimalField(decimal_places=4, max_digits=12, verbose_name='amount refund')),
                ('customer_balance', models.DecimalField(decimal_places=4, default=0, help_text='resulting customer balance.', max_digits=12, verbose_name='customer balance')),
                ('supplier_balance', models.DecimalField(decimal_places=4, default=0, help_text='resulting provider balance.', max_digits=12, verbose_name='provider balance')),
                ('operation_type', models.CharField(choices=[('customer', 'operation on customer account'), ('provider', 'operation on provider account')], default='customer', max_length=10, verbose_name='operation type')),
                ('external_desc', models.CharField(blank=True, max_length=255, verbose_name='public description')),
                ('internal_desc', models.CharField(blank=True, max_length=255, verbose_name='internal description')),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='companys', to='pyfb_company.Company', verbose_name='company')),
            ],
            options={
                'db_table': 'pyfb_company_balance_history',
                'verbose_name': 'company balance history',
                'ordering': ('company', '-created'),
                'verbose_name_plural': 'company balance history',
            },
        ),
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('account_number', models.TextField(max_length=100, verbose_name='account number')),
                ('credit_limit', models.DecimalField(decimal_places=4, default=0, help_text='Credit limit for postpaid account.', max_digits=12, verbose_name='credit limit')),
                ('low_credit_alert', models.DecimalField(decimal_places=4, default='10', help_text='Low credit limit alert.', max_digits=12, verbose_name='low credit level alert')),
                ('max_calls', models.PositiveIntegerField(default=1, help_text='maximum simultaneous calls allowed for this customer account.', verbose_name='max simultaneous calls')),
                ('calls_per_second', models.PositiveIntegerField(default=10, help_text='maximum calls per seconds allowed for this customer account.', verbose_name='max calls per second')),
                ('customer_enabled', models.BooleanField(default=True, verbose_name='customer enabled / disabled')),
                ('customer_balance', models.DecimalField(decimal_places=6, default=0, help_text='Actual customer balance.', max_digits=12, verbose_name='customer balance')),
                ('company', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='customers', to='pyfb_company.Company')),
            ],
            options={
                'db_table': 'pyfb_customer',
                'verbose_name': 'customer',
                'ordering': ('company',),
                'verbose_name_plural': 'customers',
            },
        ),
        migrations.CreateModel(
            name='Provider',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('supplier_enabled', models.BooleanField(default=True, verbose_name='provider enabled / disabled')),
                ('company', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='providers', to='pyfb_company.Company')),
            ],
            options={
                'db_table': 'pyfb_provider',
                'verbose_name': 'provider',
                'ordering': ('company',),
                'verbose_name_plural': 'providers',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.contrib import messages
from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import Company, Customer, Provider, CompanyBalanceHistory

class CompanyAdminForm(forms.ModelForm):

    class Meta:
        model = Company
        fields = '__all__'


class CompanyAdmin(admin.ModelAdmin):
    save_on_top = True
    form = CompanyAdminForm
    list_display = ['name', 'contact_name', 'contact_phone']
    readonly_fields = ['slug', 'created', 'modified']
    search_fields = ['^name', ]

admin.site.register(Company, CompanyAdmin)


class CustomerAdminForm(forms.ModelForm):

    class Meta:
        model = Customer
        fields = '__all__'


class CustomerAdmin(admin.ModelAdmin):
    # form = CustomerAdminForm
    list_display = ['company', 'customer_balance', 'customer_balance_update', 'credit_limit', 'blocking_credit_limit', 'max_calls', 'customer_enabled']
    readonly_fields = ['customer_balance', 'created', 'modified']

admin.site.register(Customer, CustomerAdmin)


class ProviderAdminForm(forms.ModelForm):

    class Meta:
        model = Provider
        fields = '__all__'


class ProviderAdmin(admin.ModelAdmin):
    #form = ProviderAdminForm
    list_display = ['company', 'supplier_balance', 'provider_balance_update', 'modified', 'supplier_enabled']
    readonly_fields = ['supplier_balance', 'created', 'modified']

admin.site.register(Provider, ProviderAdmin)


class CompanyBalanceHistoryAdminForm(forms.ModelForm):

    class Meta:
        model = CompanyBalanceHistory
        fields = '__all__'


class CompanyBalanceHistoryAdmin(admin.ModelAdmin):
    # form = CompanyBalanceHistoryAdminForm
    list_display_links = ('company', )
    list_display = ('company',
                    'amount_debited',
                    'amount_refund',
                    'customer_balance',
                    'supplier_balance',
                    'operation_type',
                    'external_desc',
                    'modified')
    ordering = ('-modified',
                'company')
    search_fields = ['^company__name',
                     '^external_desc']

    def save_model(self, request, obj, form, change):
        if change:
            messages.info(request, _(u"no need to update balance"))
        else:
            company = form.cleaned_data['company']
            amount_debited = form.cleaned_data['amount_debited']
            amount_refund = form.cleaned_data['amount_refund']
            if form.cleaned_data['operation_type'] == "customer":
                balance = Customer.objects.get(company=company)
                balance.customer_balance = balance.customer_balance - amount_debited + amount_refund
                balance.save()
                obj.customer_balance = balance.customer_balance
            elif form.cleaned_data['operation_type'] == "provider":
                balance = Provider.objects.get(company=company)
                balance.supplier_balance = balance.supplier_balance - amount_debited + amount_refund
                balance.save()
                obj.supplier_balance = balance.supplier_balance
            else:
                pass
            messages.success(request, _(u"balance updated"))
        obj.save()

    fieldsets = (
        (_(u'general'), {
            'fields': ('company',
                       'operation_type',
                       'external_desc',
                       'internal_desc')}),
        (_(u'amount'), {
            'fields': ('amount_debited',
                       'amount_refund')}),
        (_(u'balances'), {
            'fields': ('customer_balance',
                       'supplier_balance')}),
    )

    def get_list_filter(self, request):
        if request.user.is_superuser:
            return ['company', 'operation_type']
        else:
            return []

    def has_change_permission(self, request, obj=None):
        if request.user.is_superuser:
            return True
        else:
            return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_actions(self, request):
        if request.user.is_superuser:
            return
        else:
            return

    def get_readonly_fields(self, request, obj=None):
        if obj:  # This is the case when obj is already created. it's an edit
            return ['company',
                    'amount_debited',
                    'amount_refund',
                    'customer_balance',
                    'supplier_balance',
                    'operation_type']
        else:
            return ['customer_balance', 'supplier_balance']

admin.site.register(CompanyBalanceHistory, CompanyBalanceHistoryAdmin)



from . import models
from . import serializers
from rest_framework import viewsets, permissions


class CompanyViewSet(viewsets.ModelViewSet):
    """ViewSet for the Company class"""

    queryset = models.Company.objects.all()
    serializer_class = serializers.CompanySerializer
    permission_classes = [permissions.IsAuthenticated]


class CustomerViewSet(viewsets.ModelViewSet):
    """ViewSet for the Customer class"""

    queryset = models.Customer.objects.all()
    serializer_class = serializers.CustomerSerializer
    permission_classes = [permissions.IsAuthenticated]


class ProviderViewSet(viewsets.ModelViewSet):
    """ViewSet for the Provider class"""

    queryset = models.Provider.objects.all()
    serializer_class = serializers.ProviderSerializer
    permission_classes = [permissions.IsAuthenticated]


class CompanyBalanceHistoryViewSet(viewsets.ModelViewSet):
    """ViewSet for the CompanyBalanceHistory class"""

    queryset = models.CompanyBalanceHistory.objects.all()
    serializer_class = serializers.CompanyBalanceHistorySerializer
    permission_classes = [permissions.IsAuthenticated]



# Generated by Django 2.1.4 on 2020-09-14 16:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pyfb_company', '0004_auto_20181210_1138'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='blocking_credit_limit',
            field=models.BooleanField(default=True, verbose_name='block calls if credit limit is reached'),
        ),
    ]



# Generated by Django 2.1.4 on 2020-12-23 07:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pyfb_company', '0006_auto_20200917_1708'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='customer_balance_update',
            field=models.BooleanField(default=True, verbose_name='customer balance is updated after each call'),
        ),
        migrations.AddField(
            model_name='provider',
            name='provider_balance_update',
            field=models.BooleanField(default=False, verbose_name='provider balance is updated after each call'),
        ),
        migrations.AlterField(
            model_name='company',
            name='customer_balance_update',
            field=models.BooleanField(default=True, verbose_name='customer balance is updated after each call'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.urls import path, include
from rest_framework import routers

from . import api
from . import views


app_name = 'pyfb-company'

router = routers.DefaultRouter()
router.register(r'company', api.CompanyViewSet)
router.register(r'customer', api.CustomerViewSet)
router.register(r'provider', api.ProviderViewSet)
router.register(r'companybalancehistory', api.CompanyBalanceHistoryViewSet)

urlpatterns = (
    # urls for Django Rest Framework API
    path('api/v1/', include(router.urls)),
)

urlpatterns += (
    # urls for Company
    path('pyfb_company/company/', views.CompanyListView.as_view(), name='pyfb_company_company_list'),
    path('pyfb_company/company/create/', views.CompanyCreateView.as_view(), name='pyfb_company_company_create'),
    path('pyfb_company/company/detail/<slug:slug>/', views.CompanyDetailView.as_view(), name='pyfb_company_company_detail'),
    path('pyfb_company/company/update/<slug:slug>/', views.CompanyUpdateView.as_view(), name='pyfb_company_company_update'),
)

urlpatterns += (
    # urls for Customer
    path('pyfb_company/customer/', views.CustomerListView.as_view(), name='pyfb_company_customer_list'),
    path('pyfb_company/customer/create/', views.CustomerCreateView.as_view(), name='pyfb_company_customer_create'),
    path('pyfb_company/customer/detail/<int:pk>/', views.CustomerDetailView.as_view(), name='pyfb_company_customer_detail'),
    path('pyfb_company/customer/update/<int:pk>/', views.CustomerUpdateView.as_view(), name='pyfb_company_customer_update'),
)

urlpatterns += (
    # urls for Provider
    path('pyfb_company/provider/', views.ProviderListView.as_view(), name='pyfb_company_provider_list'),
    path('pyfb_company/provider/create/', views.ProviderCreateView.as_view(), name='pyfb_company_provider_create'),
    path('pyfb_company/provider/detail/<int:pk>/', views.ProviderDetailView.as_view(), name='pyfb_company_provider_detail'),
    path('pyfb_company/provider/update/<int:pk>/', views.ProviderUpdateView.as_view(), name='pyfb_company_provider_update'),
)

urlpatterns += (
    # urls for CompanyBalanceHistory
    path('pyfb_company/companybalancehistory/', views.CompanyBalanceHistoryListView.as_view(), name='pyfb_company_companybalancehistory_list'),
    path('pyfb_company/companybalancehistory/create/', views.CompanyBalanceHistoryCreateView.as_view(), name='pyfb_company_companybalancehistory_create'),
    path('pyfb_company/companybalancehistory/detail/<int:pk>/', views.CompanyBalanceHistoryDetailView.as_view(), name='pyfb_company_companybalancehistory_detail'),
    path('pyfb_company/companybalancehistory/update/<int:pk>/', views.CompanyBalanceHistoryUpdateView.as_view(), name='pyfb_company_companybalancehistory_update'),
)



# Generated by Django 2.1.4 on 2020-12-23 07:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pyfb_company', '0007_auto_20201223_0145'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='customer_balance',
            field=models.DecimalField(decimal_places=6, default=0, help_text='actual customer balance.', max_digits=12, verbose_name='customer balance'),
        ),
        migrations.AddField(
            model_name='provider',
            name='supplier_balance',
            field=models.DecimalField(decimal_places=6, default=0, help_text='actual provider balance.', max_digits=12, verbose_name='provider balance'),
        ),
    ]



# Generated by Django 2.2.17 on 2020-12-23 13:08

from django.db import migrations


def company_balance(apps, schema_editor):
    '''
    take value of customer_balance field in company model  and fill the customer_balance in customer models
    '''
    Customer = apps.get_model('pyfb_company', 'Customer')
    for customer in Customer.objects.all():
        customer.customer_balance = customer.company.customer_balance
        customer.customer_balance_update = customer.company.customer_balance_update
        customer.save()

    Provider = apps.get_model('pyfb_company', 'Provider')
    for provider in Provider.objects.all():
        provider.supplier_balance = provider.company.supplier_balance
        provider.provider_balance_update = provider.company.provider_balance_update
        provider.save()


class Migration(migrations.Migration):

    dependencies = [
        ('pyfb_company', '0008_auto_20201223_0759'),
    ]

    operations = [
        migrations.RunPython(company_balance),
    ]



# Generated by Django 2.2.17 on 2020-12-23 13:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pyfb_company', '0009_auto_20201223_1308'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='company',
            name='customer_balance',
        ),
        migrations.RemoveField(
            model_name='company',
            name='customer_balance_update',
        ),
        migrations.RemoveField(
            model_name='company',
            name='provider_balance_update',
        ),
        migrations.RemoveField(
            model_name='company',
            name='supplier_balance',
        ),
    ]
